const osrm = require('osrm');
const fs = require('fs');
const neatCsv = require('neat-csv');
const polyline = require('@mapbox/polyline');
const path = require('path');


exports.csv = async (path, separator='\t', headers=null) => {
    return await neatCsv(await _readFile(path), separator=separator, headers=headers);
}

exports.getOSRM = (path,algorithm='MLD')=>{
    return new osrm({
        path: path,
        algorithm: algorithm,
    });
}

exports.osrmRoute = _osrmRoute;
function _osrmRoute(osrmClient, coords){
    return new Promise((resolve, reject) => {
        osrmClient.route({
            coordinates: coords
        }, function(err, result) {
            if (err) {
                reject(err)
                return
            }
            resolve(result);
        });
    });
}

exports.osrmTable = (osrmClient, coords, multiplier=1) =>{
    return new Promise((resolve, reject) => {
        osrmClient.table({
            coordinates: coords
        }, function(err, result) {
            if (err) {
                reject(err)
                return
            }
            let width = result.durations.length;
            let height = result.durations[0].length;
            for (let i = 0; i < width; i++) {
                for (let j = 0; j < height; j++) {
                    result.durations[i][j] = Math.floor(result.durations[i][j] * multiplier);
                }
            }
            resolve(result);
        });
    });
}

exports.shell = (cmd)  => {
    const exec = require('child_process').exec;
    return new Promise((resolve, reject) => {
        exec(cmd, (error, stdout, stderr) => {
            if (error) {
                console.warn(error);
                reject(error);
                return;
            }
            resolve(stdout ? stdout : stderr);
        });
    });
}

exports.readFile = _readFile;
exports.writeFile = _writeFile;

function _readFile(fileName, type='utf8'){
    return new Promise(function(resolve, reject) {
        fs.readFile(fileName, type, (err, data) => {
            err ? reject(err) : resolve(data);
        });
    });
}

function _writeFile(fileName, content){
    return new Promise(function(resolve, reject) {
        fs.writeFile(fileName, content, 'utf8', (err) => {
            if (err) {
                console.log(err);
                throw err;
            }
            resolve();
        });
    });
}

exports.route2csv = _route2csv;
async function _route2csv(route, osrmapi, namer=(id)=>{return `${id}`;}){
    let lines = [];
    lines.push('ID,Type,Location,Arrival,Departure,Driving Distance')
    let lastLoc = null;
    for (let i = 0 ; i < r.steps.length; i ++){
        step = r.steps[i];
        let curLoc = step.location;
        let distance = 0;
        if (curLoc && lastLoc){
            distance = (await _osrmRoute(osrmapi, [curLoc, lastLoc])).routes[0].distance;
        }
        lastLoc = curLoc;
        let name = null;
        let arrv = _formatTime(step.arrival);
        let dept = arrv;
        let locStr = '';
        if (step.location){
            locStr = [step.location[1],step.location[0]].join(',');
        }
        if (step.type == 'start' || step.type == 'end'  || step.type == 'break'){
            name = step.type;
        }else{
            name = namer(step.job);
            dept = _formatTime(step.arrival + step.service);
        }
        lines.push(`"${name}","${step.type}","${locStr}","${arrv}","${dept}","${distance}"`);
    }
    return lines.join('\n');
}

exports.route2html = _route2html;
async function _route2html(route, osrmapi, describer = (coord)=>{return `${coord}`;}){
    let coords = [];
    let cache = {};
    route.steps.forEach((step, whocares) => {
        if (step.type != 'break'){
            if (! (step.location in cache)){
                coords.push(step.location);
            }
            cache[step.location] = true;
        }
    });
    let geometry = (await _osrmRoute(osrmapi,coords)).routes[0].geometry;
    let decoded = polyline.decode(geometry);
    let pl= [];
    decoded.forEach((d,i) => {
        pl.push([d[1],d[0]]);
    });
    let features = [];
    coords.forEach((coord, index) => {
        features.push({
            type: 'Feature',
            properties:{
                label: `S${index}`,
                description: describer(coord),
                icon: 'shop',
            },
            geometry:{
                type: 'Point',
                coordinates: coord,
            },
        });
    })

    let template = await _readFile(path.resolve(__dirname, 'template.html'));
    template = template.replace('{{ data }}',JSON.stringify({
        places:{
            type: 'FeatureCollection',
            features: features,
        },
        path: pl,
    },null,4));
    return template;
}

function _formatTime(seconds){
    d = new Date('2000-01-01 00:00:00');
    d.setSeconds(d.getSeconds()+seconds);
    hh = `${d.getHours()}`;
    mm = `${d.getMinutes()}`;
    return `${hh.padStart(2,'0')}:${mm.padStart(2,'0')}`;
}
